package com.ruoyi.system.kernel;

import com.ruoyi.system.domain.dto.UpdateBaseParams;
import com.ruoyi.common.core.utils.file.FileUtils;
import com.ruoyi.system.domain.dto.DeploySSLParams;
import com.ruoyi.system.api.model.LoginUser;
import com.ruoyi.common.core.utils.ServletUtils;
import com.ruoyi.common.core.utils.StringUtils;
import com.ruoyi.system.domain.SnServer;
import org.springframework.transaction.annotation.Transactional;
import com.ruoyi.system.domain.SnServerLog;
import com.ruoyi.system.enums.TaskTypeEnum;
import com.ruoyi.system.domain.SnServerTask;
import com.ruoyi.system.domain.SnTemplate;
import com.ruoyi.system.domain.SnServerTemplate;
import org.springframework.beans.factory.annotation.Value;
import com.ruoyi.system.mapper.SnServerLogMapper;
import com.ruoyi.system.mapper.SnServerTaskMapper;
import org.springframework.beans.factory.annotation.Autowired;
import com.ruoyi.common.security.service.TokenService;
import org.springframework.stereotype.Service;

@Service
public class CreateShellTaskKernel
{
    @Autowired
    private TokenService tokenService;
    @Autowired
    private SnServerTaskMapper snServerTaskMapper;
    @Autowired
    private SnServerLogMapper snServerLogMapper;
    @Value("${dbSync.open}")
    public Boolean dbSyncOpen;
    
    @Transactional
    public SnServerTask installClient(final SnServerTemplate serverTemplate, final SnTemplate template) {
        final SnServerTask task = new SnServerTask(serverTemplate);
        task.setTaskName("安装客户端模板【" + template.getName() + "】");
        task.setType(TaskTypeEnum.installClient.getType());
        this.snServerTaskMapper.insertSnServerTask(task);
        final String domain = serverTemplate.getDomain();
        final SnServerLog log = new SnServerLog(task);
        log.setLogId((Long)null);
        log.setOptName("下载模板包");
        log.setCommand(String.format("sudo curl %s -o /root/nginx/projects/%s.tar --create-dirs", template.getPackUrl(), domain));
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("下载nginx配置");
        log.setCommand(String.format("sudo curl %s -o /root/nginx/nginx_conf/%s.conf --create-dirs", serverTemplate.getNginxCnf(), domain));
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("解压模板到项目目录");
        log.setCommand(String.format("sudo mkdir -p /root/nginx/projects/%1$s && sudo tar -xf /root/nginx/projects/%1$s.tar -C /root/nginx/projects/%1$s", domain));
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("重载nginx配置");
        log.setCommand("sudo docker exec -it sync-nginx nginx -s reload");
        this.snServerLogMapper.insertSnServerLog(log);
        return task;
    }
    
    @Transactional
    public SnServerTask deleteClient(final SnServer server, final SnServerTemplate serverTemplate) {
        final SnServerTask task = new SnServerTask(serverTemplate);
        task.setTaskName("删除客户端模板【" + serverTemplate.getTemplateName() + "】");
        task.setType(TaskTypeEnum.deleteClient.getType());
        final String domain = serverTemplate.getDomain();
        if (StringUtils.isEmpty(domain) || domain.contains("*")) {
            task.setStatus(Integer.valueOf(4));
            task.setResult("此域名为空或者包含特殊字符，不能执行删除命令");
        }
        this.snServerTaskMapper.insertSnServerTask(task);
        if (task.getStatus() != 0) {
            return null;
        }
        final SnServerLog log = new SnServerLog(task);
        final String delSql = "delete from la_user_config where type = 1 and domain = '" + domain + "';";
        log.setLogId((Long)null);
        log.setOptName("删除客户端配置");
        log.setCommand("sudo docker exec -it sync-mysql mysql -uroot -p" + server.getDeDbPsd() + " -D sync-client -e \"" + delSql + "\"");
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("删除前端包");
        log.setCommand(String.format("sudo rm -rf /root/nginx/projects/%s.tar", domain));
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("删除前端nginx配置");
        log.setCommand(String.format("sudo rm -rf /root/nginx/nginx_conf/%s.conf", domain));
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("删除项目目录");
        log.setCommand(String.format("sudo rm -rf /root/nginx/projects/%s", domain));
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("重载nginx配置");
        log.setCommand("sudo docker exec -it sync-nginx nginx -s reload");
        this.snServerLogMapper.insertSnServerLog(log);
        return task;
    }
    
    @Transactional
    public SnServerTask changeBaseDomain(final SnServer server, final String oldDomain) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        final SnServerTask task = new SnServerTask(server, loginUser);
        task.setTaskName("更换基础环境域名 " + oldDomain + " => " + server.getDomain());
        task.setType(TaskTypeEnum.changeClientDomain.getType());
        this.snServerTaskMapper.insertSnServerTask(task);
        this.insertChangeDomainLog(task, oldDomain, server.getDomain(), server.getNginxCnf());
        return task;
    }
    
    @Transactional
    public SnServerTask changeClientDomain(final SnServerTemplate serverTemplate, final String oldDomain) {
        final SnServerTask task = new SnServerTask(serverTemplate);
        task.setTaskName("更换客户端【" + serverTemplate.getTemplateName() + "】域名 " + oldDomain + " => " + serverTemplate.getDomain());
        task.setType(TaskTypeEnum.changeClientDomain.getType());
        this.snServerTaskMapper.insertSnServerTask(task);
        this.insertChangeDomainLog(task, oldDomain, serverTemplate.getDomain(), serverTemplate.getNginxCnf());
        return task;
    }
    
    public void insertChangeDomainLog(final SnServerTask task, final String oldDomain, final String domain, final String nginxCnf) {
        final SnServerLog log = new SnServerLog(task);
        log.setLogId((Long)null);
        log.setOptName("下载新域名nginx配置");
        log.setCommand(String.format("sudo curl %s -o /root/nginx/nginx_conf/%s.conf --create-dirs", nginxCnf, domain));
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("更改项目目录名称");
        log.setCommand(String.format("sudo mv /root/nginx/projects/%s /root/nginx/projects/%s", oldDomain, domain));
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("更改项目压缩包名称");
        log.setCommand(String.format("sudo mv /root/nginx/projects/%s.tar /root/nginx/projects/%s.tar", oldDomain, domain));
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("删除老域名nginx配置");
        log.setCommand(String.format("sudo rm -rf /root/nginx/nginx_conf/%s.conf", oldDomain));
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("重载nginx配置");
        log.setCommand("sudo docker exec -it sync-nginx nginx -s reload");
        this.snServerLogMapper.insertSnServerLog(log);
    }
    
    public SnServerTask deploySSL(final SnServer server, final DeploySSLParams params) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        final SnServerTask task = new SnServerTask(server, loginUser);
        task.setTaskName("部署基础环境ssl证书");
        task.setType(TaskTypeEnum.deploySSL.getType());
        task.setProcess(params.getProcess());
        this.snServerTaskMapper.insertSnServerTask(task);
        this.insertDeploySSLLog(task, params, server.getDomain(), server.getNginxCnf());
        return task;
    }
    
    public SnServerTask deploySSL(final SnServerTemplate template, final DeploySSLParams params) {
        final SnServerTask task = new SnServerTask(template);
        task.setTaskName("部署客户端域名ssl证书 " + template.getDomain());
        task.setType(TaskTypeEnum.deploySSL.getType());
        task.setProcess(params.getProcess());
        this.snServerTaskMapper.insertSnServerTask(task);
        this.insertDeploySSLLog(task, params, template.getDomain(), template.getNginxCnf());
        return task;
    }
    
    public void insertDeploySSLLog(final SnServerTask task, final DeploySSLParams params, final String domain, final String nginxCnf) {
        final SnServerLog log = new SnServerLog(task);
        if (!params.getCacheSsl() && params.getType() == 1) {
            log.setLogId((Long)null);
            log.setOptName("检查/安装 certbot");
            log.setCommand("sudo apt install -y certbot || sudo yum install -y certbot");
            this.snServerLogMapper.insertSnServerLog(log);
            log.setLogId((Long)null);
            log.setOptName("生成ssl证书");
            log.setCommand(String.format("sudo certbot certonly --webroot -w /root/nginx/projects/%1$s -d %1$s --email 1602798048@qq.com --agree-tos", domain));
            this.snServerLogMapper.insertSnServerLog(log);
            log.setLogId((Long)null);
            log.setOptName("复制证书到nginx目录");
            log.setCommand(String.format("sudo cp -r /etc/letsencrypt/archive/%s /root/nginx/ssl", domain));
            this.snServerLogMapper.insertSnServerLog(log);
        }
        else {
            final String name = FileUtils.getName(params.getSslPem());
            log.setLogId((Long)null);
            log.setOptName("下载SSL证书");
            log.setCommand(String.format("sudo curl %s -o /root/nginx/ssl/%s/%s --create-dirs", params.getSslPem(), domain, name));
            this.snServerLogMapper.insertSnServerLog(log);
            final String name2 = FileUtils.getName(params.getSslKey());
            log.setLogId((Long)null);
            log.setOptName("下载SSL密匙");
            log.setCommand(String.format("sudo curl %s -o /root/nginx/ssl/%s/%s --create-dirs", params.getSslKey(), domain, name2));
            this.snServerLogMapper.insertSnServerLog(log);
        }
        log.setLogId((Long)null);
        log.setOptName("下载带SSL的nginx配置");
        log.setCommand(String.format("sudo curl %s -o /root/nginx/nginx_conf/%s.conf --create-dirs", nginxCnf, domain));
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("重载nginx配置");
        log.setCommand("sudo docker exec -it sync-nginx nginx -s reload");
        this.snServerLogMapper.insertSnServerLog(log);
        if (!params.getCacheSsl() && params.getType() == 1) {
            log.setLogId((Long)null);
            log.setOptName("缓存证书文件");
            String command = "sudo cat " + params.getSslPemRoute() + " && sudo cat " + params.getSslKeyRoute();
            command = command.replace("/etc/ssl", "/root/nginx/ssl");
            log.setCommand(command);
            this.snServerLogMapper.insertSnServerLog(log);
        }
    }
    
    public SnServerTask updateUI(final SnServer server, final UpdateBaseParams params) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        final SnServerTask task = new SnServerTask(server, loginUser);
        task.setClientVersion(server.getVersion());
        task.setBizVersion(server.getUiVersion());
        task.setTaskName("更新基础环境UI包版本 " + params.getVersion());
        task.setType(TaskTypeEnum.updateBaseUI.getType());
        this.snServerTaskMapper.insertSnServerTask(task);
        this.insertUpdateUILog(task, params, server.getDomain());
        return task;
    }
    
    public SnServerTask updateUI(final SnServerTemplate template, final UpdateBaseParams params) {
        final SnServerTask task = new SnServerTask(template);
        task.setBizVersion(template.getVersion());
        task.setTaskName("更新客户端【" + template.getTemplateName() + "】版本 " + params.getVersion());
        task.setType(TaskTypeEnum.updateClient.getType());
        this.snServerTaskMapper.insertSnServerTask(task);
        this.insertUpdateUILog(task, params, template.getDomain());
        return task;
    }
    
    public void insertUpdateUILog(final SnServerTask task, final UpdateBaseParams params, final String domain) {
        final SnServerLog log = new SnServerLog(task);
        log.setLogId((Long)null);
        log.setOptName("下载最新UI包");
        log.setCommand(String.format("sudo curl %s -o /root/nginx/projects/%s.tar --create-dirs", params.getPackUrl(), domain));
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("解压UI包到项目目录");
        log.setCommand(String.format("sudo mkdir -p /root/nginx/projects/%1$s && sudo tar -xf /root/nginx/projects/%1$s.tar -C /root/nginx/projects/%1$s", domain));
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("重载nginx配置");
        log.setCommand("sudo docker exec -it sync-nginx nginx -s reload");
        this.snServerLogMapper.insertSnServerLog(log);
    }
    
    public SnServerTask updateBack(final SnServer server, final UpdateBaseParams params) {
        final LoginUser loginUser = this.tokenService.getLoginUser(ServletUtils.getRequest());
        final SnServerTask task = new SnServerTask(server, loginUser);
        task.setClientVersion(server.getVersion());
        task.setBizVersion(server.getBackVersion());
        task.setTaskName("更新【" + params.getServeName() + "】版本 " + (StringUtils.isNotEmpty(params.getVersion()) ? params.getVersion() : ""));
        task.setType(TaskTypeEnum.updateBack.getType());
        this.snServerTaskMapper.insertSnServerTask(task);
        final SnServerLog log = new SnServerLog(task);
        log.setLogId((Long)null);
        log.setOptName("下载docker-compose");
        log.setCommand("sudo curl https://file.movfen.com/sync-cloud/base/" + params.getServeName() + "/docker-compose.yml -o /root/" + params.getServeName() + "/docker-compose.yml --create-dirs");
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("删除容器");
        log.setCommand("sudo docker rm -f " + params.getServeName());
        this.snServerLogMapper.insertSnServerLog(log);
        log.setLogId((Long)null);
        log.setOptName("删除容器");
        log.setCommand("sudo docker rmi storehouse.xuntian-software.com/library/" + params.getServeName() + ":latest");
        this.snServerLogMapper.insertSnServerLog(log);
        if (this.dbSyncOpen && params.getServeName().equals("sync-client")) {
            log.setLogId((Long)null);
            log.setOptName("基础表结构同步");
            log.setCommand("");
            this.snServerLogMapper.insertSnServerLog(log);
        }
        log.setLogId((Long)null);
        log.setOptName("重新拉包运行容器");
        log.setCommand("cd /root/" + params.getServeName() + " && sudo docker compose up -d && cd /root");
        this.snServerLogMapper.insertSnServerLog(log);
        return task;
    }
}
